import java.net.*;
import java.io.*;

/**
 * Almost the same as the RegistrationClient of exercise 5!
 */
public class QuoteOfTheDayService extends Thread {
	/* Networking variables */
	protected Socket socket = null; // TCP connection with the server
	protected InputStream in = null; // The stream to write on
	protected OutputStream out = null; // The stream to read from

	/* Command line parameters */
	protected InetAddress serverAddress; // The server address (dcg or bluebottle)
    protected String name; // Your own (real) name
    protected int port; // Your free port

	public QuoteOfTheDayService(InetAddress serverAddress, int port) {
		super("QuoteOfTheDay Service");

		this.serverAddress = serverAddress;
		this.name = "SERVICE:QotD";
		this.port = port;

		Messenger messenger = new Messenger(port); // Receiving requests and sending QotDs
		messenger.start();
	}

	/**
	 * Invoked automatically by the jvm before deleting the object ("destructor").
	 */
    protected void finalize() throws IOException {
		closeConnection();
    }

	/**
	 * Opens the connection with the server.
	 */
	protected void openConnection() throws IOException {
		socket = new Socket(serverAddress,NetworkUtilities.SERVERPORT);
		in = socket.getInputStream();
		out = socket.getOutputStream();
	}

	/**
	 * Closes the connection with the server.
	 */
	protected void closeConnection() throws IOException {
		socket.close();
	}

	/**
	 * The main-loop, opens a connection, registers with the server, receives
	 * the user list and closes the connection repeatedly.
	 */
    public void run() {
		while (true) {
			try {
				/* Open connection */
				openConnection();


				/* Registers itself with the server */
				registerUser();

				/* Receives and prints the user list */
				receiveUserList();


				/* Close connection */
				closeConnection();

				/* sleep some time -> should be in a variable... (config?) */
				Thread.sleep(60 * 1000);
			} catch (IOException ioe) {
				System.out.println("Oops, we caused an IOError. Stopping.");
				ioe.printStackTrace();
				break;
			} catch (InterruptedException ie) {
				/* Interrupted from sleeping -> wake up and do your work */
				continue;
			} catch (Exception e) {
				System.out.println("Something went wrong. Stopping.");
				e.printStackTrace();
				break;
			}
		}
		System.out.println("Registration client done.");
    }

	/**
 	 * Sends the following information to register with the server (number
 	 * of bytes given in parenthesis):
	 * type of message: REGISTER (1), own port (2), length of name (1),
	 * own name (var <= 255).
	 */
    protected void registerUser() throws IOException {
		out.write(NetworkUtilities.REGISTER); // write REGISTER command
		NetworkUtilities.write16(out,port); // write port, using write16
		out.write(name.length()); // write length of name
		out.write(name.getBytes(),0,name.length()); // write name as byte array
		out.flush(); // flush the stream
	}

	/**
	 * The server sends the list of all active clients (number of bytes given
	 * in parenthesis):
	 * type of message: USERLIST (1), number of users (2), list of users,
	 * where each user is defined as: length of name (1), name (var <= 255),
	 * ip (4), port (2). The ip is given with most significant byte first, ie.
	 * the address 129.132.130.158 is sent/received with 129 as first byte,
	 * 132 second...
	 */
	protected void receiveUserList() throws IOException {
		byte type = (byte)in.read();
		if (type != NetworkUtilities.USERLIST) { // compare received and expected type
			System.out.println("Unrecognized protocol message.");
			throw new IOException("Unrecognized protocol message.");
		}
		int number = NetworkUtilities.read16(in);
		for (int i=0; i<number; i++) {
			int nameLength = in.read(); // read the length of the name
			byte[] nameBuffer = new byte[nameLength];
			in.read(nameBuffer); // read the name as byte array
			String name = new String(nameBuffer); // convert byte array to string
			byte[] ip = new byte[4];
			in.read(ip); // read ip as byte array
			int port = NetworkUtilities.read16(in); // read the port number
			// Well... we don't need the user list ;-)...
		}
	}

	/*------------------------------------*/

    /**
     * Invoked by the jvm to start the program.
     */
    public static void main(String[] arguments) {
		if (arguments.length != 2) {
			printUsage();
		} else {
			try {
				/* Parsing the command line parameters */
				InetAddress ip = InetAddress.getByName(arguments[0]);
				int port = Integer.parseInt(arguments[1]);

				/* Starting the client */
				QuoteOfTheDayService qotdService = new QuoteOfTheDayService(ip,port);
				qotdService.start();
			} catch (Exception e) {
				System.out.println("Ooops, something went wrong.");
				e.printStackTrace();
				System.out.println();
				printUsage();
			}
		}
    }

	/**
	 * Prints some information about the usage of this program/its arguments.
	 */
	private static void printUsage() {
		System.out.println("Usage: java QuoteOfTheDayService <server-address> <port>");
		System.out.println("Where");
		System.out.println("<server-address> is either 'dcg.ethz.ch' or 'dcg2.ethz.ch' (try both!),");
		System.out.println("<port> is a free port number on your machine > 1023, eg. '6789'");
		System.exit(0);
	}
}